#include <iostream>

#include <string.h>

using namespace std;

// Datovy typ struct je prevzat z jazyka C a rozsiren do stejne
// funkcnosti jako trida class. Oproti verzi v C, muze obsahovat
// aktivni casti kodu - metody. Od tridy se lisi predevsim
// vychozimi pristupovymi pravy. Struktura ma defaultne public.
struct Text{
// Defaultni nastaveni, neni treba to udavat rucne:
//public:

	// Vsechny datove slozky jsou v pameti ulozene primo za sebou
    int dstMac;
    int srcMac;
    short int proto;
    char b;
    char c;
    int i;

    // Kazda trida a struktura ma svuj konstruktor. Muze
    // jich mit vice, ale musi se lisit poctem nebo typem
    // parametru. Konstruktor je volan vzdy pri vytvoreni
    // instance objektu. Pokud neni konstruktor uveden,
    // prekladac vytvori prazdny bezparametricky konstruktor.

    // Povsimnete si zvlaste inicializace datove slozky i.
    Text(char _c, int _i): i(_i) {
		b = 'a';
		c = _c;
    }

    // Nepovinou, ale uzitecnou casti je destruktor. Je volan
    // vzdy, kdyz je instance objektu odstranovana z pameti.
    // Pokud provadime v objektu nejakou alokaci pameti, nebo
    // jinych zdroju, je jejich uvolneni vhodne provest zde.
    ~Text(){
    }
};


class Trida{
// Od struct se lisi defaultnimi pristupovymi pravy:
//private:  - netreba uvadet

    // Spatne, referencni promenna nemuze existovat bez
	// inicializace. Protoze inicializace v objektech
	// probiha v konstruktoru, nemuze trida ani struktura
	// obsahovat referencni datovou slozku!!
    //int &ref;

	// Privatni promenna. Nelze k ni pristupovat pomoci
	// operatoru . nebo ->
    int data1;
public:
    // Specialni typ metody, nedela nic jineho, nez ze
    // zpristupnuje privatni promennou.
    int getData1(){
    	return data1;
    }
};

// Ukazka tvorby jednoducheho objektu pro ulozeni textu.
class String{
    char *text;
    int velikost;

    // Staticka promenna je sdilena vsemi instancemi objektu
    // typu String. Je potreba ji definovat mimo telo objektu.
    // (viz. kod hned za Stringem) Je to kvuli realizaci
    // prekladacem - instance objektu nemusi existovat, tedy
    // neni nikde v pameti, ale staticka promenna musi byt
    // v pameti dostupna. Proto se definuje samostatne.
    static int count;
public:
    String(const char *_text){
		// Nezadouci, jedna se o melkou kopii
		//text = _text;

    	// Alokujeme pamet na novy text o velikosti
    	// delka textu + jeden byte na ukoncovaci znak.
		text = new char[strlen(_text) + 1];
		// Spravne, vytvarime hlubokou kopii:
		strcpy(text, _text);
		velikost = strlen(_text);

		// Datova slozka nam urcuje, kolik instanci objektu
		// typu string je aktualne v bezicim programu.
		count++;
    }
    ~String(){
    	count --;

    	// V destruktoru po sobe musime uklidit.
    	delete [] text;
    }
    // Staticka (tridni) metoda muze pracovat pouze ze statickymi
    // promennymi. Je to vlastne funkce umistena do tridy nebo
    // struktury. Tyto metody lze volat i bez instance objektu
    // (viz dale)
    static int getCount(){
		return count;
    }
};

// Pokud neuvedeme nasledujici radek, lze kod prelozit, ale linker
// nezvladne vyslednou aplikaci sestavit, protoze nenajde promennou
// count. Pokud je pouze soucasti deklarace slozeneho datoveho typu
// neni v pameti fyzicky pritomna. Nasledujici radek zajisti, aby
// tento problem nenastal.
int String::count;

// Tridy nebo struktury nemusi byt deklarovany najednou a na jednom
// miste. V nekterych pripadech je zadouci kod rozdelit.

// Oznamujeme prekladaci, ze se nekde v kodu nachazi trida Cislo.
class Cislo;

// Podobnych oznameni muzeme udelat vice.
class Cislo;

// Nyni deklarujeme pouze vnitrni strukturu. Takze vsechny
// datove slozky a metody a jejich parametry.
class Cislo{
    int cislo;
public:
    Cislo(int _cislo);
};

// Samotne metody lze definovat pozdeji, na jinem miste.
// Pokud neni jejich kod v programu primo pouzity, nemusi
// se definice dokoncu vubec uvadet.
Cislo::Cislo(int _cislo){
    cislo = _cislo;
}

// Nasleduji 3 funkce, ktere maji za ukol prohodit hodnoty ve
// svych dvou parametrech.

// Spatne! Funkce meni jen hodnoty lokalnich promennych a a b.
// Puvodni promenne, ktere byly predany jako paramtry zustaly
// nezmeneny - jedna se o predani paramtru hodnotou!!!
void swap1(int a, int b){
    int c = a;
    a = b;
    b = c;
}

// Spravne reseni je predavat parametry misto hodnotou odkazem.
// Ukazka pomoci ukazatelu:
void swap2(int *a, int *b){

    int c = *a;
    *a = *b;
    *b = c;
}

// A jeste verze pomoci referenci:
void swap3(int &a, int &b){
    int c = a;
    a = b;
    b = c;
}

// Defaultni hodnoty parametru
int fce1(int a, int b = 0, int c = 0){
	return 0;
}

// Spatne! Za prvnim defaultnim uz mohou byt pouze defaultni!
//int fce1(int a, int b = 0, int c){
//	return 0;
//}

// Funkce lze pretezovat, lze tedy pouzit stejny nazev pro
// vice funkci. Musi se vsak lisit typem nebo poctem parametru.
char fce1(char a, int b, int c);

// Nasleduje chyba, funkce nelze pri volani odlisit od fce1
// se zadanym parametrem a a defaultnim parametrem b.
// int fce1(int aa, int bb);

int minimum(int a, int b);
float minimum(float a, float b);

int main(){
    // Spatne, nelze mit referenci bez inicializace:
    //int &ref;

    String s("text");

    cout << "Pocet instanci typu String: " << String::getCount() << endl;

    // U objektu typu struct a class je jejich velikost v pameti dana
    // souctem velikosti vsech datovych slozek objektu. (Krome statickych
    // datovych slozek)
    cout << sizeof(Text) << endl;

    // Volani metod objektu na instanci objektu, na ukazateli a na referenci.
    Trida t, *p_t = &t, &r_t = t;
    cout << t.getData1() << endl;
    cout << p_t->getData1() << endl;
    cout << r_t.getData1() << endl;

    // Pokud pristupujeme k metodam, nebo datovym slozkam objektu pomoci
    // ukazatele, musime pouzit operator "->". Je to kvuli tomu, ze operator
    // "." prislusi promennym slozenych datovych typu, ale ukazatel takovym
    // typem neni. Ukazatel sam je jen adresou a nema zadne metody ani
    // datove slozky. Proto je treba tyto dva pristupy odlisit ruznymi operatory.

    return 0;
}
